import { resolve, relative } from 'pathe';
import { defu } from 'defu';
import { pascalCase, camelCase } from 'scule';
import { useNuxt, defineNuxtModule, useLogger, createResolver, tryResolveModule, addServerHandler, addImportsSources, addTemplate } from '@nuxt/kit';

async function generateTypes(endpoints, globalOpenAPIOptions) {
  let runningCount = 0;
  process.on("exit", () => {
    if (runningCount > 0)
      throw new Error("Failed to generate OpenAPI types");
  });
  const openAPITS = await interopDefault(import('openapi-typescript'));
  const schemas = await Promise.all(
    Object.entries(endpoints).map(async ([id, endpoint]) => {
      let types = "";
      const schema = await resolveSchema(endpoint);
      runningCount++;
      try {
        types = await openAPITS(schema, {
          commentHeader: "",
          ...globalOpenAPIOptions,
          ...endpoint.openAPITS
        });
      } catch {
        types = `
export type paths = Record<string, never>
export type webhooks = Record<string, never>
export type components = Record<string, never>
export type external = Record<string, never>
export type operations = Record<string, never>
        `.trimStart();
      } finally {
        runningCount--;
      }
      return `
declare module '#nuxt-api-party/${id}' {
${types.replace(/^/gm, "  ").trimEnd()}
}`.trimStart();
    })
  );
  return schemas.join("\n\n");
}
async function resolveSchema({ schema }) {
  const nuxt = useNuxt();
  if (typeof schema === "function")
    return await schema();
  if (typeof schema === "string" && !schema.match(/^https?:\/\//))
    schema = resolve(nuxt.options.rootDir, schema);
  return schema;
}
async function interopDefault(m) {
  const resolved = await m;
  return resolved.default || resolved;
}

const module = defineNuxtModule({
  meta: {
    name: "nuxt-api-party",
    configKey: "apiParty",
    compatibility: {
      nuxt: "^3"
    }
  },
  defaults: {
    endpoints: {},
    client: false,
    openAPITS: {}
  },
  async setup(options, nuxt) {
    const moduleName = "nuxt-api-party";
    const logger = useLogger(moduleName);
    const getRawComposableName = (endpointId) => `$${camelCase(endpointId)}`;
    const getDataComposableName = (endpointId) => `use${pascalCase(endpointId)}Data`;
    if (!Object.keys(options.endpoints).length && !nuxt.options.runtimeConfig.apiParty)
      logger.error("Missing any API endpoint configuration. Please check the `apiParty` module configuration in `nuxt.config.ts`.");
    nuxt.options.runtimeConfig.apiParty = defu(
      nuxt.options.runtimeConfig.apiParty,
      options
    );
    if (!nuxt.options.ssr) {
      logger.info("Enabling Nuxt API Party client requests by default because SSR is disabled.");
      options.client = "always";
    }
    const resolvedOptions = nuxt.options.runtimeConfig.apiParty;
    nuxt.options.runtimeConfig.public.apiParty = defu(
      nuxt.options.runtimeConfig.public.apiParty,
      resolvedOptions.client ? resolvedOptions : {
        // Only expose cookies endpoint option to the client
        endpoints: Object.fromEntries(
          Object.entries(resolvedOptions.endpoints).map(
            ([endpointId, endpoint]) => [endpointId, { cookies: endpoint.cookies }]
          )
        ),
        client: false
      }
    );
    const { resolve } = createResolver(import.meta.url);
    nuxt.options.build.transpile.push(resolve("runtime"));
    const relativeTo = (path) => relative(
      resolve(nuxt.options.rootDir, nuxt.options.buildDir, "module"),
      resolve(path)
    );
    const endpointKeys = Object.keys(resolvedOptions.endpoints);
    const schemaEndpoints = Object.fromEntries(
      Object.entries(resolvedOptions.endpoints).filter(([, endpoint]) => "schema" in endpoint)
    );
    const schemaEndpointIds = Object.keys(schemaEndpoints);
    const hasOpenAPIPkg = await tryResolveModule("openapi-typescript", [nuxt.options.rootDir]);
    if (schemaEndpointIds.length && !hasOpenAPIPkg) {
      logger.warn("OpenAPI types generation is enabled, but the `openapi-typescript` package is not found. Please install it to enable endpoint types generation.");
      schemaEndpointIds.length = 0;
    }
    addServerHandler({
      route: "/api/__api_party/:endpointId",
      method: "post",
      handler: resolve("runtime/server/handler")
    });
    nuxt.hook("nitro:config", (config) => {
      config.externals ||= {};
      config.externals.inline ||= [];
      config.externals.inline.push(...[
        resolve("runtime/utils"),
        resolve("runtime/formData"),
        resolve("runtime/server/$api")
      ]);
      config.alias ||= {};
      config.alias[`#${moduleName}/server`] = resolve(nuxt.options.buildDir, `module/${moduleName}-nitro`);
      config.virtual ||= {};
      config.virtual[`#${moduleName}/server`] = () => `
import { _$api } from '${resolve("runtime/server/$api")}'
${endpointKeys.map((i) => `
export const ${getRawComposableName(i)} = (...args) => _$api('${i}', ...args)
`.trimStart()).join("")}`.trimStart();
      if (schemaEndpointIds.length) {
        config.typescript ||= {};
        config.typescript.tsConfig ||= {};
        config.typescript.tsConfig.include ||= [];
        config.typescript.tsConfig.include.push(`./module/${moduleName}-schema.d.ts`);
      }
      config.imports = defu(config.imports, {
        presets: [{
          from: `#${moduleName}/server`,
          imports: endpointKeys.map((i) => getRawComposableName(i))
        }]
      });
    });
    addImportsSources({
      from: resolve(nuxt.options.buildDir, `module/${moduleName}.mjs`),
      imports: endpointKeys.flatMap((i) => [getRawComposableName(i), getDataComposableName(i)])
    });
    nuxt.options.alias[`#${moduleName}`] = resolve(nuxt.options.buildDir, `module/${moduleName}`);
    addTemplate({
      filename: `module/${moduleName}.mjs`,
      getContents() {
        return `
import { _$api } from '${resolve("runtime/composables/$api")}'
import { _useApiData } from '${resolve("runtime/composables/useApiData")}'
${endpointKeys.map((i) => `
export const ${getRawComposableName(i)} = (...args) => _$api('${i}', ...args)
export const ${getDataComposableName(i)} = (...args) => _useApiData('${i}', ...args)
`.trimStart()).join("")}`.trimStart();
      }
    });
    addTemplate({
      filename: `module/${moduleName}.d.ts`,
      getContents() {
        return `
// Generated by ${moduleName}
import type { $Api, $OpenAPI, ApiFetchOptions } from '${relativeTo("runtime/composables/$api")}'
import type { UseApiData, UseOpenAPIData, UseApiDataOptions } from '${relativeTo("runtime/composables/useApiData")}'

${schemaEndpointIds.map((i) => `
import type { paths as ${pascalCase(i)}Paths } from '#${moduleName}/${i}'
`.trimStart()).join("").trimEnd()}

export type { $Api, $OpenAPI, ApiFetchOptions, UseApiData, UseOpenAPIData, UseApiDataOptions }

${endpointKeys.map((i) => `
export declare const ${getRawComposableName(i)}: ${schemaEndpointIds.includes(i) ? `$OpenAPI<${pascalCase(i)}Paths>` : "$Api"}
export declare const ${getDataComposableName(i)}: ${schemaEndpointIds.includes(i) ? `UseOpenAPIData<${pascalCase(i)}Paths>` : "UseApiData"}
`.trimStart()).join("").trimEnd()}
`.trimStart();
      }
    });
    addTemplate({
      filename: `module/${moduleName}-nitro.d.ts`,
      async getContents() {
        return `
// Generated by ${moduleName}
export { ${endpointKeys.map((i) => getRawComposableName(i)).join(", ")} } from './${moduleName}'
`.trimStart();
      }
    });
    if (schemaEndpointIds.length) {
      addTemplate({
        filename: `module/${moduleName}-schema.d.ts`,
        async getContents() {
          return await generateTypes(schemaEndpoints, resolvedOptions.openAPITS);
        }
      });
      nuxt.hook("prepare:types", ({ references }) => {
        references.push({ path: resolve(nuxt.options.buildDir, `module/${moduleName}-schema.d.ts`) });
      });
    }
  }
});

export { module as default };
